@@ -6,6 +6,8 @@ use std::rc::Rc;
 use std::sync::Arc;
 
 use anyhow::Context as _;
+use cargo_util_schemas::manifest::RustVersion;
+use cargo_util_schemas::manifest::{TomlManifest, TomlProfiles};
 use semver::Version;
 use serde::ser;
 use serde::Serialize;
@@ -18,8 +20,7 @@ use crate::core::{Dependency, PackageId, PackageIdSpec, SourceId, Summary};
 use crate::core::{Edition, Feature, Features, WorkspaceConfig};
 use crate::util::errors::*;
 use crate::util::interning::InternedString;
-use crate::util::toml::schema::{TomlManifest, TomlProfiles};
-use crate::util::{short_hash, Config, Filesystem, RustVersion};
+use crate::util::{short_hash, Config, Filesystem};
 
 pub enum EitherManifest {
     Real(Manifest),
