@@ -3,11 +3,13 @@
 //! [`HttpRegistry`]: super::http_remote::HttpRegistry
 //! [`RemoteRegistry`]: super::remote::RemoteRegistry
 
+use crate::util::interning::InternedString;
 use anyhow::Context;
 use cargo_credential::Operation;
 use cargo_util::registry::make_dep_path;
 use cargo_util::Sha256;
 
+use crate::core::global_cache_tracker;
 use crate::core::PackageId;
 use crate::sources::registry::MaybeLock;
 use crate::sources::registry::RegistryConfig;
@@ -34,6 +36,7 @@ const CHECKSUM_TEMPLATE: &str = "{sha256-checksum}";
 pub(super) fn download(
     cache_path: &Filesystem,
     config: &Config,
+    encoded_registry_name: InternedString,
     pkg: PackageId,
     checksum: &str,
     registry_config: RegistryConfig,
@@ -50,6 +53,13 @@ pub(super) fn download(
     if let Ok(dst) = File::open(path) {
         let meta = dst.metadata()?;
         if meta.len() > 0 {
+            config.deferred_global_last_use()?.mark_registry_crate_used(
+                global_cache_tracker::RegistryCrate {
+                    encoded_registry_name,
+                    crate_filename: pkg.tarball_name().into(),
+                    size: meta.len(),
+                },
+            );
             return Ok(MaybeLock::Ready(dst));
         }
     }
@@ -106,6 +116,7 @@ pub(super) fn download(
 pub(super) fn finish_download(
     cache_path: &Filesystem,
     config: &Config,
+    encoded_registry_name: InternedString,
     pkg: PackageId,
     checksum: &str,
     data: &[u8],
@@ -115,6 +126,13 @@ pub(super) fn finish_download(
     if actual != checksum {
         anyhow::bail!("failed to verify the checksum of `{}`", pkg)
     }
+    config.deferred_global_last_use()?.mark_registry_crate_used(
+        global_cache_tracker::RegistryCrate {
+            encoded_registry_name,
+            crate_filename: pkg.tarball_name().into(),
+            size: data.len() as u64,
+        },
+    );
 
     cache_path.create_dir()?;
     let path = cache_path.join(&pkg.tarball_name());
